# File rvsao/Fquot/fqrslts.x
# August 3, 1994
# By Gerard Kriss (June 1987), Johns Hopkins University
# Modified by Doug Mink, Harvard-Smithsonian Center for Astrophysics
 
# Produce a statistical summary of all the Fourier quotient fits
# to the object spectrum.  All input is in common in "rvsao.com"
 
include	"rvsao.h"
 
procedure fqrslts()
 
bool	uniform
bool	good[MAXTEMPS]
char	stat
int	i, j
real	sigavg, sigdisp, sigwavg, sigwdisp, savgerr, smaxerr
real	gamavg, gamdisp, gamwavg, gamwdisp, gavgerr, gmaxerr
real	disp
 
include	"rvsao.com"
include	"fqres.com"
include	"results.com"
 
begin
 
# Qualify results as acceptable first.
 
	do i = 1, ntemp {
	    if ((cz[i] > CZMIN  &&  cz[i] < CZMAX ) &&
		(sig[i] > SIGMIN && sig[i] < SIGMAX) &&
		(gam[i] > GAMMIN && gam[i] < GAMMAX) &&
		(perdeg[i] > CHIMIN && perdeg[i] < CHIMAX) &&
		(abs (czerr[i] / cz[i]) > ERRMIN) &&
		(abs (sigerr[i] / sig[i]) > ERRMIN) &&
		(abs (gamerr[i] / gam[i]) > ERRMIN) &&
		(abs (czerr[i] / cz[i] )< ERRMAX) &&
		(abs (sigerr[i] / sig[i]) < ERRMAX) &&
		(abs (gamerr[i] / gam[i]) < ERRMAX) )
		good[i] = true
	    else
		good[i] = false
	    }
 
# Compute statistical information on the results.
	uniform = true
	call mean (ntemp, cz, czerr, good, uniform, czavg, czdisp)
	call mean (ntemp, sig, sigerr, good, uniform, sigavg, sigdisp)
	call mean (ntemp, gam, gamerr, good, uniform, gamavg, gamdisp)
 
	uniform = false
	call mean (ntemp, cz, czerr, good, uniform, czwavg, czwdisp)
	call mean (ntemp, sig, sigerr, good, uniform, sigwavg, sigwdisp)
	call mean (ntemp, gam, gamerr, good, uniform, gamwavg, gamwdisp)
 
	uniform = true
	call mean (ntemp, czerr, czerr, good, uniform, cavgerr, disp)
	call mean (ntemp, sigerr, sigerr, good, uniform, savgerr, disp)
	call mean (ntemp, gamerr, gamerr, good, uniform, gavgerr, disp)
 
	cmaxerr = 0.
	smaxerr = 0.
	gmaxerr = 0.
	do i = 1, ntemp {	# Find largest error on each
	    if (good[i]) {
		if (czerr[i] > cmaxerr)
		    cmaxerr = czerr[i]
		if (sigerr[i] > smaxerr)
		    smaxerr = sigerr[i]
		if (gamerr[i] > gmaxerr)
		    gmaxerr = gamerr[i]
		}
	    }
 
# Set up heading in log files
	do i = 1, nlogfd {
	    call fprintf(logfd[i], "\nResults for object:\t%s\n\n")
		call pargstr(specname)
	    call fprintf(logfd[i],
	  "Template\t     CZ      \t Dispersion \t Line Strength \t Chi-Square\n")
	    }
 
# Record results in log files
 
	do i = 1, ntemp {
	    stat = '-'
	    if ( good[i] )
		stat = '+'
	    do j = 1, nlogfd {
		call fprintf(logfd[j],
		"%c%1d  %s\t%5.0f +- %4.0f\t%4.0f +- %4.0f\t%5.2f +- %5.2f\t%8.2f\n")
		    call pargc (stat)
		    call pargi (i)
		    call pargstr (tempname[1,i])
		    call pargd (cz[i])
		    call pargd (czerr[i])
		    call pargd (sig[i])
		    call pargd (sigerr[i])
		    call pargd (gam[i])
		    call pargd (gamerr[i])
		    call pargr (perdeg[i])
		}
	    }
 
# Record statistical summary in the log files
 
	do i = 1, nlogfd {
	    call fprintf(logfd[i], "\nStatistical summary for object: %s\n\n")
		call pargstr(specname)
	    call fprintf(logfd[i], "Parameter\tMean \t\tWeighted Mean ")
	    call fprintf(logfd[i],"\t\tAvg Error \tLargest Error\n\n")
	    call fprintf(logfd[i], "CZ       \t%6.0f +- %4.0f")
		call pargr (czavg)
		call pargr (czdisp)
	    call fprintf(logfd[i],"\t%6.0f +- %4.0f\t\t%4.0f  \t\t%4.0f\n")
		call pargr (czwavg)
		call pargr (czwdisp)
		call pargr (cavgerr)
		call pargr (cmaxerr)
 
	    call fprintf(logfd[i], "Dispersion\t%6.0f +- %4.0f")
		call pargr (sigavg)
		call pargr (sigdisp)
	    call fprintf(logfd[i],"\t%6.0f +- %4.0f\t\t%4.0f  \t\t%4.0f\n")
		call pargr (sigwavg)
		call pargr (sigwdisp)
		call pargr (savgerr)
		call pargr (smaxerr)
 
	    call fprintf(logfd[i], "Line Strength\t%6.3f +- %5.3f")
		call pargr (gamavg)
		call pargr (gamdisp)
	    call fprintf(logfd[i],"\t%6.3f +- %5.3f\t\t%5.3f  \t\t%5.3f\n")
		call pargr (gamwavg)
		call pargr (gamwdisp)
		call pargr (gavgerr)
		call pargr (gmaxerr)
	    }
end
 

#  Calculate means and dispersions for data arrays

procedure mean (npts, points, errs, qual, uniform, avg, disp)
 
int	npts			# Number of points
double	points[ARB], errs[ARB]	# Array of data and errors
bool	qual[ARB]		# Quality flags for points
bool	uniform			# Flag for weighting scheme
real	avg, disp		# Returned mean and dispersion
 
int	i
real	weight, var, norm
 
begin
 
	var = 0.
	avg = 0.
	norm = 0.
	do i = 1, npts {
	    if (qual[i]) {
		if ( uniform )
		    weight = 1.
		else if ( errs[i] != 0. )
		    weight = 1. / errs[i]
		else
		    weight = 1.e10
		norm = norm + weight
		avg = avg + weight * points[i]
		var = var + weight * points[i]**2
		}
	    }
	if (norm != 0.) {
	    avg = avg / norm
	    var = var / norm - avg**2
	    }
	else {
	    avg = avg / 1.e-10
	    var = var / 1.e-10 - avg**2
	    }
	disp = sqrt (var)
 
end
# Dec  2 1992	Change name to fqrslts congruent with other RVSAO tasks

# Apr 13 1994	Clean up code after running ftnchk
# Apr 22 1994	Clean up code after running spplint
# Aug  3 1994	Change common and header from fquot to rvsao
