# File rvsao/Fquot/t_fquot.x
# October 6, 1995
# By Gerard Kriss, Johns Hopkins University
# Modified by Doug Mink, Harvard-Smithsonian Center for Astrophysics
 
# An IRAF task for obtaining redshifts and velocity dispersions using the
# Fourier Quotient method.  Adapted from (and uses parts of) a program
# originally written by Paul Schechter in FORTRAN.

include	<imio.h>
include	<imhdr.h>
include	<fset.h>
include "rvsao.h"
include "xcv.h"
include "shdr.h"
 
procedure t_fquot ()
 
char	specfiles[SZ_PATHNAME]	# List of object spectra
char	specfile[SZ_PATHNAME]	# Object spectrum file name
char	tempfiles[SZ_PATHNAME]	# List of template spectra
char	tempfile[SZ_PATHNAME]	# Template spectrum file name
char	temppath[SZ_PATHNAME]	# Template spectrum path name
char	tempdir[SZ_PATHNAME]	# Directory for template spectra
char	tempnums[SZ_LINE]	# List of multispec spectra to read
#char	filt_type[SZ_FNAME]	# Filter for transform
char    chopstr[SZ_FNAME]
pointer	wlspec		# Wavelength vector for input spectrum
pointer	wltemp		# Wavelength vector for template overlap
pointer	shspec		# Rebinned object spectrum in overlap region
pointer	shtemp		# Rebinned template spectrum in overlap region
int	i, j, fd
int	stat, col
int	speclist	# List of input spectra
int	nfiles		# Number of object spectrum files
int	ntfiles		# Number of template spectrum files
int	ntspec		# Number of template multispec spectra
int	templist	# List of template spectra
int	zlist		# List of redshift guesses
int	logfiles	# list of log files
char	logfile[SZ_PATHNAME] # Log file name
char	str[SZ_LINE]
bool	renorm		# Renormalization flag
double	tmstrt		# Starting template wavelength after zguess applied
double	tmfnsh		# Ending template wavelength after zguess applied
real	strtwav0, finwav0	# Wavelength limits, if entered
real	spmean, speci, sptot	# Mean for renormalization
double  logstrt, logfin		# Log10 start and end overlap wavelengths
double  strtwav, finwav		# Start and end correlation wavelengths
double	minfin, maxstrt		# Start and stop overlap wavelengths
double  spshft, tmshft		# Spectrum shifts in log wavelength
int	mspec_range[3,MAX_RANGES]
int	mspec		# Object spectrum to read from multispec file 
int	tspec_range[3,MAX_RANGES]
int	tspec		# Template spectrum to read from multispec file
int	itemp,nsp
int	junk
int	ispec, nspec
int	ntx		# Number of templates actually used
double	dval
pointer spectrum	# Object spectrum
pointer specim		# Object image header structure
pointer specsh		# Object spectrum header structure
pointer tempspec	# Template spectrum
pointer tempim		# Template image header structure
pointer tempsh		# Template spectrum header structure
char	zchar[SZ_FNAME]

int	decode_ranges(),get_next_number()
int	fntopnb(),fntlenb(), fntgfnb(), imaccess(), strlen(), lpath
bool	clgetb()
int	clpopnu(), clgeti(), open(), clgfil(), ctod(), strdic()
real	clgetr()
int	nsmooth
char	wtitle[16]
 
include	"rvsao.com"
include	"results.com"
include	"fqres.com"
include	"fquot.com"
 
begin
 
	strtwav0 = 0.d0
	finwav0 = 0.d0
	call sprintf (wtitle,16,"Wavelength")

# Get task parameters.

# Get list of spectra to cross-correlate
	call clgstr ("spectra",specfiles,SZ_PATHNAME)
	speclist = fntopnb (specfiles,YES)
	nfiles = fntlenb (speclist)

# Multspec spectrum numbers (use only first if multiple files)
	call clgstr ("specnum",specnums,SZ_LINE)
	if (specnums[1] == EOS)
	    call strcpy ("0",specnums,SZ_LINE)
	if (decode_ranges (specnums, mspec_range, MAX_RANGES, nmspec) == ERR)
	    call error (1, "T_XCSAO: Illegal multispec list")

# Number of object spectra to process
	if (nfiles > 1)
	    nspec = nfiles
	else
	    nspec = nmspec
 
# Get list of templates against which to correlate spectra
	call clgstr ("templates",tempfiles,SZ_PATHNAME)
	call clgstr ("tempdir",tempdir,SZ_PATHNAME)
	templist = fntopnb (tempfiles,YES)
	ntfiles = fntlenb (templist)
	call fntclsb (templist)

# Multspec template numbers (use only first if multiple files)
	call clgstr ("tempnum",tempnums,SZ_LINE)
	if (decode_ranges (tempnums, tspec_range, MAX_RANGES, ntspec) == ERR)
	    call error (1, "T_XCSAO: Illegal multispec list")

# Number of object spectra to process
	if (ntfiles > 1)
	    ntemp = ntfiles
	else
	    ntemp = ntspec

# Set switches for intermediate data plots
	pltspec = clgetb ("obj_plot")
	pltcon = clgetb ("window_plot")
	pltfft = clgetb ("fft_plot")
	pltcor = clgetb ("fit_plot")
	debug = clgetb ("debug")

# Emission line chopping flag
	call clgstr ("s_emchop",chopstr,SZ_FNAME)
	sfchop = strdic (chopstr, chopstr, SZ_FNAME, XC_CTYPES)
	call clgstr ("t_emchop",chopstr,SZ_FNAME)
	tfchop = strdic (chopstr, chopstr, SZ_FNAME, XC_CTYPES)

# Choose whether or not to write TDC archive record
	savevel = FALSE
	savevel = clgetb ("save_vel")
 
	zlist = clpopnu ("zlist")
	sig0 = clgetr ("sigma")
	gam0 = clgetr ("gamma")
 
# Set fraction of spectrum to apodize at each end
	han = clgetr ("bell_window")

	cpf = clgetr ("counts_per_photon")
	lo = clgeti ("low_bin")
	nrun = clgeti ("nrun")

# Continuum fit parameter pset
	call cont_get_pars()

# Number of times to smooth displayed spectrum
	nsmooth = 10
	nsmooth = clgeti ("nsmooth")

# Choose whether or not to renormalize data
	renorm = clgetb ("renormalize")

	chi0 = clgetr ("initial_chi")
	unc = clgetr ("converge")
	niter = clgeti ("max_iterations")

# If npts not a power of two, determine next largest power of two:
	npts = clgeti ("ncols")
	if (npts != 256 && npts != 512 && npts != 1024 && npts != 2048 &&
	    npts != 4096 && npts != 8192 ) {
	    npts = alog (real (npts)) / LN2 +  1.
	    npts = 2 ** npts
	    }

# Allocate spectrum, correlation, and x-axis buffers
	call malloc (shspec, npts, TY_REAL)
	call malloc (shtemp, npts, TY_REAL)
	call malloc (wltemp, npts, TY_REAL)
 
	logfiles = clpopnu("logfiles")
 
# Open log files and write a header.
	call fseti(STDOUT, F_FLUSHNL, YES)
	i = 0
	call strcpy ("RVSAO.FQUOT",taskname,SZ_LINE)
	while (clgfil(logfiles, logfile, SZ_PATHNAME) != EOF ) {
	    fd = open(logfile, APPEND, TEXT_FILE)
	    if (fd == ERR) break
	    call loghead (taskname,str)
	    call fprintf(fd, "%s\n")
		call pargstr(str)
	    i = i + 1
	    logfd[i] = fd
	    }
	nlogfd = i
	call clpcls(logfiles)
 
	# Loop through the list of spectra and get the answers
	do ispec = 1, nspec {

	# Get next object spectrum file name from the list
	    if (nfiles > 1 || ispec == 1)
		junk = fntgfnb (speclist, specfile, SZ_PATHNAME)

	# Get next multispec number from list
	    if (nfiles == 1 || ispec == 1)
		junk = get_next_number (mspec_range, mspec)

	# Check for writability if results are being saved in image header
	    if (savevel) {
		if (imaccess (specfile, READ_WRITE) == NO) {
		    call eprintf ("FQUOT: cannot write to %s; not saving results\n")
			call pargstr (specfile)
		    savevel = FALSE
		    }
		}

	# Check for readability of object spectrum
	    if (imaccess (specfile, READ_ONLY) == NO) {
		call eprintf ("FQUOT: cannot read spectrum file %s \n")
		    call pargstr (specfile)
		next
		}

	# Load spectrum
	    call getspec (specfile, mspec, spectrum, specim, specsh)
	    if (specim == ERR) break

	    do j = 1, nlogfd {
		call fprintf(logfd[j]," OBJECT: %s \n\n")
		    call pargstr(specname)
		}

	    dlogw = spdwl
	    deltav = dlogw * CLN10
	    if (renorm) {
		sptot = 0.d0
		nsp = 0
		do i = 1, specpix {
		    speci = Memr[spectrum+i-1]
		    if (speci != 0.) {
			sptot = sptot + speci
			nsp = nsp + 1
			}
		    }
		spmean = sptot / double (nsp)
		do i = 1, specpix {
		    if (Memr[spectrum+i-1] != 0.) {
			Memr[spectrum+i-1] = Memr[spectrum+i-1] / spmean
			}
		    }
		}

	# Set up wavelength vector for spectrum graph
	    wlspec = SX(specsh)

	# If plot enabled, show the object spectrum.
	    if (pltspec) {
		call plotspec (specpix,Memr[spectrum],specname,
			       Memr[wlspec],wtitle,nsmooth)
		}

	# Get initial redshift estimate
	    stat =  clgfil (zlist, zchar, SZ_FNAME)
	    col = 1
	    stat = ctod (zchar, col, dval)
	    z0 = dval
 
	    if (debug) {
		do j = 1, nlogfd {
		    call fprintf (logfd[j]," Object: %s\n")
			call pargstr (specname)
		    }
		}

	# Loop through templates doing fits to each one.
	    templist = fntopnb (tempfiles,YES)
	    ntx = 0
	    do itemp = 1, ntemp {

	    # Initialize template results
		cz[itemp] = 0.d0
		czerr[itemp] = 0.d0
		sig[itemp] = 0.d0
		sigerr[itemp] = 0.d0
		gam[itemp] = 0.d0
		gamerr[itemp] = 0.d0

	    # Get next template spectrum file name from the list
		if (ntfiles > 1 || itemp == 1)
		    junk = fntgfnb (templist, tempfile, SZ_PATHNAME)

	    # Get next multispec number from list
		if (ntfiles == 1 || itemp == 1)
		    junk = get_next_number (tspec_range, tspec)

	    # Check for readability of template spectrum
		call strcpy (tempdir,temppath,SZ_PATHNAME)
		lpath = strlen (temppath)
		if (temppath[lpath] != '/') {
		    temppath[lpath+1] = '/'
		    temppath[lpath+2] = EOS
		    }
		call strcat (tempfile,temppath,SZ_PATHNAME)
		if (imaccess (temppath, READ_ONLY) == NO) {
		    call eprintf ("XCSAO: cannot read template file %s \n")
			call pargstr (temppath)
		    break
		    }

	    # Load template spectrum
		call gettemp (tempfile,tspec,tempdir,tempspec,tempim,
			      tempsh,itemp)
		if (tempim == ERR) break
		ntx = ntx + 1
		tmstrt = testrt[itemp]
		tmfnsh = tefnsh[itemp]

		if (debug) {
		    do j = 1, nlogfd {
			call fprintf(logfd[j],"\n Template: %s %f-%f\n")
			    call pargstr(tempname[1,itemp])
			    call pargd(tmstrt)
			    call pargd(tmfnsh)
			}
		    }

	    # Spectrum emission line chopping flag
		schop = FALSE
		switch (sfchop) {
		    case CHOP:
			schop = TRUE
		    case NOCHOP:
			schop = FALSE
		    case TEMPFILE:
			call imgbpar (tempim,"CHOPEM",schop)
		    case SPECFILE:
			call imgbpar (specim,"CHOPEM",schop)
		    }

	    # Template emission line chopping flag
		tchop = FALSE
		switch (tfchop) {
		    case CHOP:
			tchop = TRUE
		    case NOCHOP:
			tchop = FALSE
		    case TEMPFILE:
			call imgbpar (tempim,"CHOPEM",tchop)
		    case SPECFILE:
			call imgbpar (specim,"CHOPEM",tchop)
		    }

	    # Figure lambda and log-lambda limits and increment
		maxstrt = max (spstrt, tmstrt)
		minfin = min (spfnsh, tmfnsh)
		if ((strtwav0 == 0.0) || (strtwav0 < 10.**(maxstrt)))
		    strtwav = 10.**(maxstrt)
		else
		    strtwav = strtwav0
		if ((finwav0 == 0.0) || (finwav0 > 10.**(minfin)))
		    finwav = 10.**(minfin)
		else
		    finwav = finwav0
		logstrt = dlog10 (strtwav)
		logfin = dlog10 (finwav)
		twl1[itemp] = strtwav
		twl2[itemp] = finwav

	    # Fill in globally available values of wavelength
		wave0 = strtwav
		logw0 = logstrt
		dlogw = (logfin - logstrt) / double (npts - 1)
		deltav = dlogw * CLN10

		if (debug) {
		    call printf("FQUOT: wave0 = %8.3f, from %9.7f by %8.7f, npts = %d \n")
			call pargd (wave0)
			call pargd (logw0)
			call pargd (dlogw)
			call pargi (npts)
		    }

	    # Rebin spectrum to overlap this template
		if (debug) {
		    call printf("FQUOT: spectrum %9.7f to %9.7f = %d \n")
			call pargd (spstrt)
			call pargd (spfnsh)
			call pargi (specpix)
		    }
		spshft = 0.d0
		call rebinl (Memr[spectrum], specsh, spshft,
			     Memr[shspec], npts, logw0, dlogw)

	    #  Rebin template to overlap this spectrum
		if (debug) {
		    call printf("FQUOT: template %9.7f to %9.7f = %d\n")
			call pargd (tmstrt)
			call pargd (tmfnsh)
			call pargi (temppix[itemp])
		    }
		tmshft = 0.d0
		call rebinl (Memr[tempspec], tempsh, tmshft,
			     Memr[shtemp], npts, logw0, dlogw)

	    # Set up wavelength vector for intermediate graphs
		
		do j = 1, npts {
		    Memr[wltemp+j-1] = 10 ** (logw0 + (j-1) * dlogw)
		    }

	    # Find Fourier quotient
		call fqfit (Memr[shspec],Memr[shtemp],Memr[wltemp],
			    perdeg[itemp],itemp)
 
	    # Fix problems with zero valued results
		do j = 1, 3 {
		    if (z[j] == 0.)
			z[j] = 1.e-15
		    if (ze[j] == 0.)
			ze[j] = 1.e-15
		    }

		cz[itemp] = C * (10. ** (-z[1] * dlogw) - 1.) + tempvel[itemp]
		czerr[itemp] = ze[1] * deltav
		sig[itemp] = exp( z[2] ) * deltav
		sigerr[itemp] = ze[2] * sig[itemp]
		if (sig[itemp] == 0.) {
		    sig[itemp] = 1.e-10
		    sigerr[itemp] = 1.e-10
		    }
		gam[itemp] = z[3]
		gamerr[itemp] = ze[3]
 
	    # Unmap the template image
		call close_image (tempim, tempsh)
		}

	    call fntclsb (templist)

#	Log results
	    call fqrslts ()
	
#	Save best Cz and error to image header, if requested
	    if (savevel) {
		call imaddr (specim,"CZFQ    ",czwavg)
		call imaddr (specim,"CZFQERR ",czwdisp)
		IM_UPDATE(specim) = YES
		}

#	Free spectrum image and headers
	    call close_image (specim, specsh)
	    }
 
# Close the log files
	do i = 1, nlogfd {
	    call close (logfd[i])
	    }

# Free rebinned spectra and wavelength vectors
	call mfree (wltemp, TY_REAL)
	call mfree (shtemp, TY_REAL)
	call mfree (shspec, TY_REAL)
 
	call clpcls(speclist)
	call clpcls(zlist)
 
end
 
# Mid 1970's	Paul Schechter
# June	1987	Gerard Kriss -- Changed front-end routines for IRAF use

# Sept	1989	Doug Mink -- add parameters for 2-D files
# Oct	1991	Doug Mink -- upgrade spectrum handling to match XCSAO
# Nov 20 1991	Free temporary strings before allocating them

# Feb 19 1991	Rebin overlapping regions of template and spectrum
#		Compute wavelength vectors for intermediate data graphs
# Apr 22 1992	Add capability of reading object spectrum from a multispec file
#		Make rebinned spectra arrays rather than pointers
# Jun 16 1992	Drop old polynomial continuum fit
# Dec  2 1992	Make convergence criteria double
#		Add ncols and tempnum parameters

# Jun  4 1993	Put pixel to wavelength conversion in a subroutine using MWCS
# Jun 16 1993	Set version to 1.1; use shdr for linear wavelength conversion
# Jun 30 1993	Pass spectrum header structure to rebinl
# Jul  7 1993	Add spectrum header to getspec and gettemp
# Jul  8 1993	Close sh properly
# Aug 20 1993	Deal with double wavelength vector
# Dec  3 1993	Update to version 1.2

# Feb  3 1994	Handle unreadable and unwriteable files gracefully
# Apr 12 1994	Pass MWCS header to getspec and gettemp
# Apr 26 1994	Use spectrum header wavelength vector directly
# Apr 26 1994	Avoid pointers for shorter arrays and strings
# May 23 1994	Clean up write-protect check
# Jun 23 1994	Keep MWCS pointer in SHDR structure
# Jun 24 1994	Update to version 1.4
# Aug  3 1994	Get version from rvsao.h
# Aug  3 1994	Change common and header from fquot to rvsao
# Aug  3 1994	Set emission line chopping flag for each spectrum/template
# Aug  4 1994	Move nmspec and specnums to labelled common
# Aug 10 1994	Write log heading in one line instead of two

# Jan 31 1995	Change lengths of file and directory names to sz_pathname
# Sep 21 1995	Move FQUOT parameters to fquot.com
# Oct  6 1995	Use CLOSE_IMAGE to close images
