# File rvsao/Util/gettemp.x
# July 8, 1996
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After Gerard Kriss, Johns Hopkins University

# Copyright(c) 1994 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
# GETTEMP opens one template spectrum, extracts wavelength and velocity
# information from the header, and returns the template spectrum.
# Data is shared in common in "rvsao.com"
 
include <imhdr.h>
include	"rvsao.h"
include	"shdr.h"
define  LEN_USER_AREA   2880
 
procedure gettemp (tempfile, mspec, tempdir, tempspec, tempim, tempsh, itemp)
 
char	tempfile[ARB]	# Template file name
int	mspec		# Number of spectrum to read from multispec file
char	tempdir[ARB]	# Template directory
pointer	tempspec	# Template spectrum (returned)
pointer	tempim		# Template image header structure (returned)
pointer	tempsh		# Template spectrum header structure (returned)
int	itemp		# Template number for saved information

char	temppath[SZ_PATHNAME]
int	mode, nline, npix
char	line[SZ_LINE]
double	bcvel,hcvel,dindef
char	keyword[20]
int	imaccf()
real	compbcv()
double	dlog10()

include	"rvsao.com"
include	"getim.com"

begin
	mode = READ_ONLY

	call strcpy (tempdir,temppath,SZ_PATHNAME)
	call strcat (tempfile,temppath,SZ_PATHNAME)
	if (debug) {
	    call printf ("GETTEMP: %d %s.imh\n")
		call pargi (itemp)
		call pargstr (temppath)
	    }

# Get template spectrum and image header
	call getimage (temppath, mspec, tempspec,
			tempim, tempsh, npix, tempname[1,itemp], mode, debug)

	if (tempim == ERR) return
	if (debug) {
	    call printf ("GETTEMP: %s  %d pixels read\n")
		call pargstr (tempname[1,itemp])
		call pargi (npix)
	    }

	temppix[itemp] = npix

# Set filter flag
	tempfilt[itemp] = fiflag

#  Get radial velocity and error
	tempvel[itemp] = VEL(tempsh)
	call sprintf (keyword,SZ_LINE,"APVEL%d")
	call pargi (nline)
	if (imaccf (tempim, keyword) == YES) {
	    call imgspar (tempim,keyword,line, SZ_LINE)
	    call sscan (line)
		call gargd (tempvel[itemp])
	    }
	else {
	    tempvel[itemp] = 0.d0
	    call imgdpar (tempim, "VELOCITY", tempvel[itemp])
	    }

	tempshift[itemp] = 0.d0
	call imgdpar (tempim, "TSHIFT  ", tempshift[itemp])
	dindef = INDEFD
	hcvel = dindef
	call imgdpar (tempim, "HCV     ", hcvel)
	bcvel = dindef
	call imgdpar (tempim, "BCV     ", bcvel)

# Calculate template heliocentric velocity, or use one from file

	switch (tvcor) {
	    case NONE:
		temphcv[itemp] = 0.
	    case HCV:
		temphcv[itemp] = compbcv (tempim,0,debug)
	    case BCV:
		temphcv[itemp] = compbcv (tempim,1,debug)
	    case FHCV:
		temphcv[itemp] = 0.d0
		if (hcvel != dindef)
		    temphcv[itemp] = hcvel
		else if (bcvel != dindef)
		    temphcv[itemp] = bcvel
	    case FBCV:
		temphcv[itemp] = 0.d0
		if (bcvel != dindef)
		    temphcv[itemp] = bcvel
		else if (hcvel != dindef)
		    temphcv[itemp] = hcvel
	    }

	if (debug) {
	    call printf ("GETTEMP: vel: %.3f tsh: %.3f hcv: %.3f\n")
		call pargd (tempvel[itemp])
		call pargd (tempshift[itemp])
		call pargd (temphcv[itemp])
	    }


#  Set log-wavelength limits
	tempdc[itemp] = DC(tempsh)
	testrt[itemp] = dlog10 (W0(tempsh))
	tefnsh[itemp] = dlog10 (W1(tempsh))
	if (debug) {
	    call printf ("GETTEMP: lambda %9.4f - %9.4f by %9.4f")
		call pargd (W0(tempsh))
		call pargd (W1(tempsh))
		call pargd (WP(tempsh))
	    if (DC(tempsh) != DCLOG) {
		call printf (" = %d points\n")
		    call pargi (temppix[itemp])
		}
	    else
		call printf ("\n")
	    dlogw = (tefnsh[itemp] - testrt[itemp]) / double (temppix[itemp] - 1)
	    call printf ("GETTEMP: log lambda %9.7f - %9.7f by %9.7f")
		call pargd (testrt[itemp])
		call pargd (tefnsh[itemp])
		call pargd (dlogw)
	    if (DC(tempsh) == DCLOG) {
		call printf (" = %d points\n")
		    call pargi (temppix[itemp])
		}
	    else
		call printf ("\n")
	    }
end
# May 22 1992	Do not change mspec
# Dec  2 1992	Print log wavelength limits in debug mode

# May 24 1993	Set up MWCS pixel<->wavelength transformations
# May 25 1993	Get all velocity information directly from image header
# May 25 1993	Read spectrum using getimage
# Jun  4 1993	Use onedspec spectrum header structure
# Jun 15 1993	Fix use of onedspec spectrum header
# Jun 30 1993	Return spectrum header as argument
# Jul  7 1993	Add spectrum header to getimage arguments
# Aug 20 1993	Add more diagnostic output

# Mar  9 1994	Fix bug listing wavelength incremen
# Apr 12 1994	Return MWCS header pointer
# Apr 13 1994	Fix parg calls to match types
# Apr 20 1994	Fix SSCAN call
# May  3 1994	Use no temporary storage
# Jun 15 1994	Set TEMPFILT from spectrum image header
# Jun 23 1994	Keep MWCS pointer in SHDR structure
# Aug  3 1994	Change common and header from fquot to rvsao
# Dec  7 1994	Make TEMPFILT integer instead of boolean

# Feb 27 1995	Fix null directory
# Mar 13 1995	Remove unused STRLEN function declaration
# May 10 1995	Print velocities to meters/sec
# May 15 1995	Change all sz_fname to sz_pathname, which is longer
# Jul 13 1995	Add DEBUG to COMPBCV calls

# Jul  8 1996	Pass debugging flag to getimage
