# File rvsao/Util/imgpar.x
# By Doug Mink, Center for Astrophysics
# September 25, 1995

include	<syserr.h>
include	<ctype.h>
define	IDB_ENDVALUE	30

# IMGPAR -- These subroutines do not modify the value argument if the
#           header parameter is not present

# IMGBPAR -- Get an image header parameter of type boolean.  False is returned
# if the parameter cannot be found or if the value is not true.

procedure imgbpar (im, key, bval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
bool	bval		# value returned

int	imaccf()
int	idb_findrecord()
pointer	rp

begin
	if (imaccf (im, key) == YES) {
	    if (idb_findrecord (im, key, rp) != 0)
		bval = (Memc[rp+IDB_ENDVALUE-1] == 'T')
	    }
	return
end


# IMGCPAR -- Get an image header parameter of type char.

procedure imgcpar (im, key, cval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
char	cval		# parameter value returned

int	imaccf()
pointer	sp, vp
int	ip, cctoc()
errchk	syserrs, imgstr

begin
	if (imaccf (im, key) == YES) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    if (cctoc (Memc[vp], ip, cval) == 0) {
		call printf ("IMGCPAR:  Bad data in header for keyword %s\n")
		call pargstr (key)
		cval = EOS
		}
	    call sfree (sp)
	    }
	return
end


# IMGIPAR -- Get an image header parameter of type integer.

procedure imgipar (im, key, ival)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
int	ival		# parameter value returned

int	imaccf()
pointer	sp, vp
int	itemp, ip, ctoi()
errchk	syserrs, imgstr

begin
	if (imaccf (im, key) == YES) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    if (ctoi (Memc[vp], ip, itemp) == 0) {
		call printf ("IMGIPAR:  Bad data in header for keyword %s\n")
		    call pargstr (key)
		}
	    else
		ival = itemp
	    call sfree (sp)
	    }
	return
end


# IMGLPAR -- Get an image header parameter of type long integer.

procedure imglpar (im, key, lval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
long	lval		# parameter value returned

int	imaccf()
pointer	sp, vp
int	ip, ctol()
long	ltemp
errchk	syserrs, imgstr

begin
	if (imaccf (im, key) == YES) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    if (ctol (Memc[vp], ip, ltemp) == 0) {
		call printf ("IMGLPAR:  Bad data in header for keyword %s\n")
		    call pargstr (key)
		}
	    else
		lval = ltemp
	    call sfree (sp)
	    }
	return
end


# IMGSPAR -- Get an image header string parameter

procedure imgspar (im, key, sval,lsval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
char	sval[ARB]	# parameter value returned
int	lsval		# Maximum length for sval

int	imaccf()
errchk	imgstr

begin
	if (imaccf (im, key) == YES)
	    call imgstr (im, key, sval, lsval)
	return
end


# IMGRPAR -- Get an image header parameter of type real.

procedure imgrpar (im, key, rval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
real	rval		# parameter value returned

int	imaccf()
pointer	sp, vp
int	ip, ctor()
real	rtemp
errchk	syserrs, imgstr

begin
	if (imaccf (im, key) == YES) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    if (ctor (Memc[vp], ip, rtemp) == 0) {
		call printf ("IMGRPAR:  Bad data in header for keyword %s\n")
		    call pargstr (key)
		}
	    else
		rval = rval
	    call sfree (sp)
	    }
	return
end


# IMGDPAR -- Get an image header parameter of type double floating.  If the
# named parameter is a standard parameter return the value directly,
# else scan the user area for the named parameter and decode the value.

procedure imgdpar (im, key, dval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
double	dval		# parameter value returned

int	imaccf()
pointer	sp, vp
double	dtemp
int	ip, ctod()
errchk	syserrs, imgstr

begin
	if (imaccf (im, key) == YES) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
#	    call printf ("IMGDPAR: keyword = %s, value = %s\n")
#		call pargstr (key)
#		call pargstr (Memc[vp])
	    if (ctod (Memc[vp], ip, dtemp) == 0) {
		call printf ("IMGDPAR:  Bad data in header for keyword %s\n")
		    call pargstr (key)
		}
	    else
		dval = dtemp
	    call sfree (sp)
	    }
	else {
#	    call printf ("IMGDPAR: keyword %s not found\n")
#		call pargstr (key)
	    }
	return
end


# IMGDATE -- Get a date image header string parameter as day/month/year

procedure imgdate (im, key, mm, dd, yyyy)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
int	mm, dd, yyyy

int	i, j, lstr, ix1, ix2, np
int	ctoi(), strlen(), stridx(), strldx()
int	imaccf()
char	slash, dash
pointer	vp, sp, ip, tp

begin
	if (imaccf (im, key) == YES) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    call salloc (tp, SZ_LINE, TY_CHAR)
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    lstr = strlen (Memc[vp])
	    if (lstr < 1) {
		return
		}

	    dash = '-'
	    slash = '/'
	    ix1 = stridx (slash, Memc[vp])
	    if (ix1 < 1)
	        ix1 = stridx (dash, Memc[vp])
	    ip = vp + ix1 - 1
	    np = ix1 - 1
	    call strcpy (Memc[vp], Memc[tp], np)
	    i = 1
	    if (ctoi(Memc[tp],i,j) > 0)
		dd = j
	    else
		dd = 0

	    ix2 = strldx (slash, Memc[vp])
	    if (ix2 < 1)
	        ix2 = strldx (dash, Memc[vp])
	    np = ix2 - ix1 - 1
	    call strcpy (Memc[ip+1], Memc[tp], np)
	    i = 1
	    if (ctoi(Memc[tp],i,j) > 0)
		mm = j
	    else
		mm = 0

	    ip = vp + ix2 - 1
	    call strcpy (Memc[ip+1],Memc[tp],4)
	    i = 1
	    if (ctoi(Memc[tp],i,j) > 0) {
		yyyy = j
		if (yyyy < 100)
		    yyyy = 1900 + yyyy
		}
	    else
		yyyy = 0
	    call sfree (sp)
	    }
end                                     
# Jan 10 1995	Print error messages and return default values
# Apr 10 1995	Return null string from imgcpar if error
# Apr 10 1995	Return input value unchanged from other img$par if error
# Jul  7 1995	Use IMACCF instead of IDB_FINDRECORD for generality
# Sep 25 1995	Make IMGDATE accept - as well as / for date separator
