# File rvsao/Util/juldate.x
# October 18, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

include	<imhdr.h>

# JULDATE -- Comute Julian Date at midtime of observation

procedure juldate (im,ut,dj,debug)

pointer	im		# image structure for spectrum
double	ut		# UT of observation midtime (returned)
double	dj		# Julian date of observation (returned)
bool	debug		# True for diagnostic display

int	mm, dd, yyyy, julday, igreg, ja, jm, jy
double	dindef
double	ut1, ut2, exp

begin
	dindef = INDEFD
	dj = dindef

# Get date of observation
	mm = INDEFI
	dd = INDEFI
	yyyy = INDEFI
	call imgdate (im, "DATE-OBS", mm, dd, yyyy)

# Get UT at middle of observation
	ut2 = dindef
	call imgdpar (im, "UT", ut2)
	ut1 = dindef
	call imgdpar (im,"UTOPEN", ut1)
	exp = 0.
	call imgdpar (im,"EXPOSURE", exp)
	if (exp == 0.)
	    call imgdpar (im,"EXPTIME", exp)
	if (ut1 == dindef) {
	    if (ut2 != dindef)
		ut = ut2 - (exp * 0.5 / 3600.)
	    else
		ut = 0.d0
	    }
	else if (ut2 != dindef)
	    ut = (ut1 + ut2) * 0.5
	else if (ut1 != dindef)
	    ut = ut1 + (exp * 0.5 / 3600.)
	else
	    ut = 0.d0

# Find Julian Day of observation
	if (yyyy <= 0 || mm == INDEFI || dd == INDEFI || yyyy == INDEFI)
	    return

	igreg = 15 + (31 * (10 + (12 * 1582)))

	if (mm > 2) {
	    jy = yyyy
	    jm = mm + 1
	    }
	else {
	    jy = yyyy - 1
	    jm = mm + 13
	    }

	julday = int (365.25*jy) + int (30.6001*jm) + dd + 1720995

	if (dd+31*(mm+12*yyyy) >= igreg) {
	    ja = int (0.01 * jy)
	    julday = julday + 2 - ja + int (0.25 * ja)
	    }

# Compute Julian Date of observation
	dj = double (julday) - 0.5d0 + (ut / 24.d0)
	if (debug) {
	    call printf("JULDATE: date = %d/%d/%d ut = %.6f = %h\n")
		call pargi(dd)
		call pargi(mm)
		call pargi(yyyy)
		call pargd(ut)
		call pargd(ut)
	    call printf("JULDATE: date = %.6f\n")
		call pargd (dj)
	    }

end
# Jul 13 1995	New program
# Sep 19 1995	Format floating point numbers
# Sep 22 1995	Set UT to 0 if no times are set
# Oct 18 1995	Use either EXPOSURE or EXPTIME
