# File rvsao/Util/plotspec.x
# August 10, 1994
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After Gerard Kriss, Johns Hopkins University

# Copyright(c) 1994 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

#  PLOTSPEC uses IRAF plot routines to plot a spectrum on STDGRAPH.
#  FIXIT edits a spectrum by interpolating between marked points.		
#  Adapted from splot in the IRAF onedspec package
 
include	<gset.h>
include	<pkg/gtools.h>
include	"rvsao.h"
 
procedure plotspec (npoints, pix, gtitle, xvec, xlabel, nsmooth)
 
int	npoints		# Number of points in spectrum
real	pix[ARB]	# Unsmoothed vector of points to plot
char	gtitle[ARB]	# Title for the plot
real	xvec[ARB]	# X coordinates for plotted vector
char	xlabel[ARB]	# Label for x axis
int	nsmooth		# Number of times to smooth data

int	ipix1		# first pixel to plot
int	ipix2		# last pixel to plot
int	npix		# number of points to plot
char	command[SZ_FNAME]
int	wc, key, itemp, ip
bool	newgraph, dfirst, zfirst
int	i, d1, d2, ix, iwidth, i1, i2, ix0
real	wx, wy, xmax, wx0
pointer	gt, gfd, sp
pointer	smpix		# Smoothed vector of points to plot
char    device[SZ_FNAME] # Display on which to plot data
char    plotter[SZ_FNAME] # Device on which to make hard copies
char    linebuf[SZ_LINE] # Line buffer for input
 
int	gt_gcur(), clgeti(), ctoi(), getline()
pointer	gopen(), gt_init()
 
include	"rvsao.com"
 
begin

# Open display and eliminate y-axis minor ticks.
	call clgstr("device", device, SZ_FNAME)
	gfd = gopen (device, NEW_FILE, STDGRAPH)
	If (gfd == ERR) {
	    call printf ("PLOTSPEC:  cannot open display %s\n")
		call pargstr (device)
	    return
	    }
	call gseti (gfd, G_YNMINOR, NO)
	call gseti (gfd, G_XNMINOR, YES)
	call gsetr (gfd, G_TICKLABELSIZE, 0.8)
 
# Initialize graph format
	gt = gt_init()
	call loghead (taskname,linebuf)
	call gt_seti (gt, GTSYSID, NO)
	call gt_sets (gt, GTTITLE, linebuf)
	call gt_sets (gt, GTSUBTITLE, gtitle)
	call gt_sets (gt, GTXLABEL, xlabel)

# Set up vector to hold smoothed data to be plotted
	call smark (sp)
	call salloc (smpix, npoints, TY_REAL)
	call amovr (pix,Memr[smpix],npoints)
	call smooth (Memr[smpix],npoints,nsmooth)
 
# Initial plot of data array
	ipix1 = 1
	ipix2 = npoints
	npix = npoints
	call replot (gfd, gt, npix, Memr[smpix+ipix1-1], xvec[ipix1])
	newgraph = false
	dfirst = true
	zfirst = true
 
# Handle cursor interaction
	while (gt_gcur ("cursor", wx, wy, wc, key, command, SZ_FNAME) != EOF) {

# Find closest pixel within wavelength limits
	    if (wx <= xvec[ipix1])
		ix = ipix1
	    else if (wx >= xvec[ipix2])
		ix = ipix2
	    else {
		i1 = ipix1
		i2 = ipix2 - 1
		do i = i1, i2 {
		    if (wx >= xvec[i] && wx < xvec[i+1]) {
			if (wx - xvec[i] > 0.5)
			    ix = i
			else
			    ix = i + 1
			break
			}
		    }
		}
	    switch (key) {
		case 'r':
		    newgraph = true

#	Make a hardcopy of the current display
		case '@':
		    call gflush (gfd)
		    call gclose (gfd)
		    call printf ("Making hard copy on stdplot\n")
		    call flush (STDOUT)
		    call clgstr("plotter", plotter, SZ_FNAME)
		    gfd = ERR
		    gfd = gopen (plotter, APPEND, STDGRAPH)
		    If (gfd == ERR) {
			call printf ("PLOTSPEC:  cannot open plotter %s\n")
			    call pargstr (plotter)
			next
			}
		    call replot (gfd, gt, npix, Memr[smpix+ipix1-1], xvec[ipix1])
		    call gflush (gfd)
		    call gclose (gfd)
		    gfd = gopen (device, APPEND, STDGRAPH)
		    call gseti (gfd, G_YNMINOR, NO)
		    call gseti (gfd, G_XNMINOR, YES)
		    call gsetr (gfd, G_TICKLABELSIZE, 0.8)
 
#	Print wavelength coordinate at cursor
		case 'w':
		    call gclose (gfd)
		    call printf ("Wavelength %8.2f, Pixel %d: %8.2f\n")
		    call pargr (wx)
		    call pargi (ix)
		    call pargr (wy)
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)

#	Set number of times to smooth spectrum
		case 'n':
		    call gclose (gfd)
		    call printf("Number of times to smooth spectrum (%d): ")
			call pargi (nsmooth)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			i = ctoi (linebuf,ip,itemp)
			nsmooth = itemp
			call amovr (pix,Memr[smpix],npoints)
			call smooth (Memr[smpix],npoints,nsmooth)
			}
		    gfd = gopen (device, APPEND, STDGRAPH)
		    newgraph = true

#	Delete portion of data
		case 'd':
		    if (dfirst) {
			d1 = ix
			call gclose (gfd)
			call printf("d again to delete data from pixel %d\n")
			    call pargi (d1)
			call flush (STDOUT)
			gfd = gopen (device, APPEND, STDGRAPH)
			dfirst = false
			}
		    else {
			dfirst = true
			d2 = ix
			call gclose (gfd)
			call printf("filling from pixel %d to pixel %d\n")
			    call pargi (d1)
			    call pargi (d2)
			call flush (STDOUT)
			gfd = gopen (device, APPEND, STDGRAPH)
			call fixit (npoints, pix, d1, d2)
			call amovr (pix,Memr[smpix],npoints)
			call smooth (Memr[smpix],npoints,nsmooth)
			newgraph = true
			}

#	Reinitialize two-keystroke commands
		case 'x':
		    if (!zfirst) {
			call gclose (gfd)
			call printf("zoom cancelled\n")
			call flush (STDOUT)
			gfd = gopen (device, APPEND, STDGRAPH)
			zfirst = true
			}
		    if (!dfirst) {
			call gclose (gfd)
			call printf("delete cancelled\n")
			call flush (STDOUT)
			gfd = gopen (device, APPEND, STDGRAPH)
			dfirst = true
			}

#	Zoom in on section of graph
		case 'z':
		    if (zfirst) {
			ipix1 = ix
			zfirst = false
			call gclose (gfd)
			call printf("z again to zoom from %5.0f\n")
			    call pargr (xvec[ipix1])
			call flush (STDOUT)
			}
		    else {
			ipix2 = ix
			zfirst = true
			if (ipix2 < ipix1) {
			    itemp = ipix1
			    ipix1 = ipix2
			    ipix2 = itemp
			    }
			npix = ipix2 - ipix1 + 1
			call gclose (gfd)
			call printf("zooming from %5.0f to %5.0f\n")
			    call pargr (xvec[ipix1])
			    call pargr (xvec[ipix2])
			call flush (STDOUT)
			newgraph = true
			}
		    gfd = gopen (device, APPEND, STDGRAPH)

#	Unzoom
		case 'u':
		    ipix1 = 1
		    ipix2 = npoints
		    npix = npoints
		    newgraph = true

#	Print menu
		case 'h','?',' ':
		    call gclose (gfd)
		    call printf ("@  make hard copy of screen\n")
		    call printf ("c  show x and y coordinates\n")
		    call printf ("d  delete data between 1st and 2nd positions\n")
		    call printf ("h  this menu\n")
		    call printf ("p  select correlation peak\n")
		    call printf ("q  leave plot\n")
		    call printf ("r  replot current graph\n")
		    call printf ("s  reset peak search half-width\n")
		    call printf ("u  unzoom\n")
		    call printf ("x  cancel zoom or delete after 1st char\n")
		    call printf ("z  zoom between 1st and 2nd positions\n")
		    call printf ("?  this menu\n")
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)

#	Set peak search width
		case 'S','s':
		    call gclose (gfd)
		    iwidth = clgeti ("pksrch")
		    call printf ("Half-width in pixels for peak search (%d) = \n")
			call pargi (iwidth)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			i = ctoi (linebuf,ip,iwidth)
			call clputi ("pksrch",iwidth)
			}
		    gfd = gopen (device, APPEND, STDGRAPH)
 
#	Set estimate of cross-correlation peak to the cursor value
		case 'P','p':
		    call gclose (gfd)
		    ix0 = ix
		    wx0 = wx
		    iwidth = 25
		    iwidth = clgeti ("pksrch")
		    i1 = ix - iwidth + 1
		    if (i1 < 1)
			i1 = 1
		    i2 = ix + iwidth
		    if (i2 > ipix2)
			i2 = ipix2
		    xmax = pix[ix]
		    for (i = i1; i <= i2; i = i + 1) {
			if (pix[i] >= xmax) {
			    xmax = pix[i]
			    ix = i
			    }
			}
		    z[1] = ix
		    wx = xvec[ix]
		    call printf ("Max peak at %d = %8.2f (%d = %8.2f)\n")
			call pargi (ix)
			call pargr (wx)
			call pargi (ix0)
			call pargr (wx0)
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)

		case 'Q','q':
		    break

#	Default is to print coordinates of cursor
		default:
		    call gclose (gfd)
		    call printf ("x,y: %10.3f %10.4g\n")
			call pargr (wx)
			call pargr (wy)
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)
		}
 
#	Replot graph if necessary
	    if (newgraph) {
		call replot (gfd, gt, npix, Memr[smpix+ipix1-1], xvec[ipix1])
		newgraph = false
		}
	    }
 
# Close graph window
 
	call gclear (gfd)
	call gclose (gfd)
	call gt_free (gt)
	call sfree (sp)
	return
end

 
procedure fixit (npts, pix, d1, d2)
 
int	npts		# Number of points in spectrum
real	pix[ARB]	# Array of points to edit
int	d1, d2		# Pixels to interpolate between
 
int	i, temp
real	p0, slope
 
begin
 
# If d2 < d1, switch the order.
	if ( d2 < d1 ) {
	    temp = d1
	    d1 = d2
	    d2 = temp
	    }
 
# If d2 = d1, increase it by one.
	if ( d2 == d1 )
	    d2 = d2 + 1
 
	if ( d1 > 1 && d2 < npts ) {
	    p0 = pix[d1 - 1]
	    slope = (pix[d2 + 1] - p0) / (d2 - d1)
	    }
	else if ( d1 == 1 && d2 < npts ) {
	    p0 = pix[d2 + 1]
	    slope = 0.
	    }
	else if ( d1 > 1 && d2 == npts ) {
	    p0 = pix[d1 - 1]
	    slope = 0.
	    }
 
	for ( i = d1; i <= d2; i = i + 1) {
	    pix[i] = p0 + (i - d1) * slope
	    }
end
 
# June	1987	Gerard Kriss
# Feb 	1991	Doug Mink	Clean up code
# May 	1991	Doug Mink	Add zoom and hard-copy
# Sept 	1991	Doug Mink	Separate plot devices
# Dec 18 1991	Plot using wavelength vector instead of end-points
# Dec 19 1991	Add graphics pointer argument to gflush calls

# May 28 1992	Search for maximum near peak set with 'p'

# Feb  3 1994	Add s to change width of peak search
# Mar 17 1994	Add n to smooth spectrum
# Apr  8 1994	Fix n smoothing
# Apr 11 1994	Clean up pointers before returning
# Apr 19 1994	Fix bug freeing smoothed spectrum
# Apr 20 1994	Change GETLINE from subroutine to function
# Apr 26 1994	Fix smoothed spectrum with d command, too
# May  3 1994	Add smoothed vector argument to PLOTSPEC argument list
# May  9 1994	Move smoothed vector out of argument list
# May  9 1994	Add number of times to smooth spectrum to argument list
# Aug  3 1994	Change common and header from fquot to rvsao
# Aug 10 1994	Replace system ID with task name and system ID
