# File rvsao/Util/rebin.x
# April 15, 1994
# Adapted and inverse function added by Doug Mink
# Adapted by Stephen Levine , University of Wisconsin (madraf::levine)
# from AL_REBIN in T_REBIN (in onedspec)

# REBINLOG rebins a linear spectrum to a specified range in log wavelength

include <mach.h>
include <ctype.h>
include <error.h>
include <imhdr.h>
include <fset.h>
include <math/iminterp.h>
include <math/curfit.h>

#rebinning defs	
define RB_NEAREST	1
define RB_LINEAR	2
define RB_SPLINE3	3
define RB_POLY3		4
define RB_POLY5		5
define RB_SUMS		6
define RB_FUNCTIONS	"|nearest|linear|spline3|poly3|poly5|sums|"

procedure rebinlog (imin, inpix, inwl1, indwl, imout, outpix, outwl1, outdwl)

real	imin[ARB]	# Input spectrum
int	inpix		# Number of pixels in input spectrum
double	inwl1		# Input starting wavelength
double	indwl		# Input wavelength increment
real	imout[ARB]	# Output spectrum
int	outpix		# Number of pixels in output spectrum
double	outwl1		# Output starting log wavelength (returned)
double	outdwl		# Output log wavelength increment (returned)

double	inwl2		# Input ending wavelength
double	outwl2		# Output ending log wavelength
pointer	invert, sp
char	interp_mode[SZ_LINE]
double	lw		# Log wavelength
double	wl		# Wavelength in Angstroms
double	px		# Pixel number (1=center of first)
int	mode, user_mode
int	i
#int	fd, open()
int	clgwrd()

begin
	outwl1 = log10 (inwl1)
	inwl2 = inwl1 + (indwl * double (inpix-1))
	outwl2 = log10 (inwl2)
	outdwl = (outwl2 - outwl1) / double (outpix - 1)

# Get rebinning method
	user_mode = clgwrd ("interp_mode", interp_mode, SZ_LINE, RB_FUNCTIONS)
	call cfit_mode (user_mode,mode)

# Make room for inverted solution
	call smark (sp)
	call salloc (invert, outpix+1, TY_DOUBLE)

# Compute pixel position as a function of log lambda.
	do i = 1, outpix {
	    lw = outwl1 + (outdwl * double (i - 1))
	    wl = 10.d0 ** lw
	    px = 1.d0 + (wl - inwl1) / indwl
	    Memd[invert+i-1] = px
	    }

# Interpolate input spectrum to output log lambda spectrum
	if (mode == RB_SUMS)
	    call resum (imin, imout, Memd[invert], inpix, outpix)
	else
	    call reinterp (imin, imout, Memd[invert], inpix, outpix, mode)

	call sfree (sp)

end


# REBINLIN rebins a log-wavelength spectrum to a specified linear range

procedure rebinlin (imin, inpix, inwl1, indwl, imout, outpix, outwl1, outdwl)

real	imin[ARB]	# Input spectrum
int	inpix		# Number of pixels in input spectrum
double	inwl1		# Input starting log wavelength
double	indwl		# Input log wavelength increment
real	imout[ARB]	# Output spectrum
int	outpix		# Number of pixels in output spectrum
double	outwl1		# Output starting wavelength (returned)
double	outdwl		# Output wavelength increment (returned)

double	inwl2		# Input ending log wavelength
double	outwl2		# Output ending wavelength
pointer	invert, sp
double	lw		# Log wavelength
double	wl		# Wavelength in Angstroms
double	px		# Pixel number (1=center of first)
int	mode, user_mode	# Interpolation function
char	interp_mode[SZ_LINE]
int	i
int	clgwrd()

begin
	outwl1 = 10.d0 ** inwl1
	inwl2 = inwl1 + (indwl * double (inpix-1))
	outwl2 = 10.d0 ** inwl2
	outdwl = (outwl2 - outwl1) / double (outpix - 1)

# Get rebinning method
	user_mode = clgwrd ("interp_mode", interp_mode, SZ_LINE, RB_FUNCTIONS)
	call cfit_mode (user_mode,mode)

# Make room for inverted solution
	call smark (sp)
	call salloc (invert, outpix+1, TY_DOUBLE)

# Compute pixel position as a function of log lambda.
	do i = 1, outpix {
	    wl = outwl1 + (outdwl * double (i - 1))
	    lw = log10 (lw)
	    px = 1.d0 + (lw - inwl1) / indwl
	    Memd[invert+i-1] = px
	    }

# Interpolate input spectrum to output log lambda spectrum
	if (mode == RB_SUMS)
	    call resum (imin, imout, Memd[invert], inpix, outpix)
	else
	    call reinterp (imin, imout, Memd[invert], inpix, outpix, mode)

	call sfree (sp)

end
