# File rvsao/Xcsao/xciplot.x
# August 22, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After Stephen Levine and Jon Morse

# Copyright(c) 1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

#  Plot summary page for XCSAO

include <time.h>
include	<gset.h>
include	<gio.h>
include	"rvsao.h"
define  SZ_MONTH	3
 
# Print additional information on XCSAO summary page

procedure xciplot (gp, specfile, image, mspec, strtwav, finwav)

pointer	gp		# IRAF graphics environment structure
char	specfile[ARB]	# spectrum image file name
pointer	image		# IRAF image descriptor structure
int	mspec		# Number of spectrum to read from multispec file
double	strtwav		# Starting wavelength of cross-correlation
double	finwav		# End wavelength of cross-correlation

int	j, i, mm, dd, yyyy, ntempx
int	itr[MAXTEMPS]
real	lineno
double	ra, dec, ut, dj
double	tmpvel, fracpeak
char	text[SZ_LINE+1], text1[SZ_LINE+1]
extern	rcomp()

long	clktime()
string	month	"JanFebMarAprMayJunJulAugSepOctNovDec"

include "rvsao.com"               
include "results.com"

begin                                                          

#  Set up graphics window
	call gseti (gp, G_WCS, 0)
	call gswind (gp, 0.0, 1.0, 0.0, 1.0)
	call gsview (gp, 0.0, 1.0, 0.0, 1.0)

#  Date and time of correlation
	call cnvtime (clktime(0), text1, SZ_TIME)
	call sprintf(text,SZ_LINE,"rvsao.xcsao %s %s")
	    call pargstr (VERSION)
	    call pargstr (text1)
#	call htsize (0.8)
	call gtext (gp, 0.01, 0.01, text, "f=r;q=m;s=0.6")

#  Type of peak used and the number of points
	if (pkfrac < 0.d0)
	    fracpeak = -pkfrac
	else
	    fracpeak = pkfrac
	if (fracpeak < 1.d0) {
	    call sprintf(text,SZ_LINE,"%4.2f-ht. peak fit, %d pts.")
	    call pargd (fracpeak)
	    call pargi (npmax)
	    }
	else {
	    call sprintf(text,SZ_LINE,"Peak fit with %d pts.")
	    call pargi (npmax)
	    }
	call gtext (gp, 0.35, 0.01, text, "f=r;q=m")

#  File name
	call juldate (image, ut, dj, debug)
	if (dj != INDEFD) {
	    call sprintf (text, SZ_LINE, "File: %s  JulDate: %.2f")
		call pargstr (specid)
		call pargd (dj)
	    }
	else {
	    call sprintf (text, SZ_LINE, "File: %s")
		call pargstr (specfile)
	    }
#	call htsize (0.8)
	call gtext (gp, 0.025, 0.96, text, "f=r;q=m;s=0.8")
	
#  Object, RA, Dec
	call imgdpar (image, "RA", ra)
	call imgdpar (image, "DEC", dec)
	if ((ra != INDEFR) && (dec != INDEFR)) {
	    call sprintf(text, SZ_LINE,
		"Object:%-10.10s RA: %011.2h DEC: %011.2h")
		call pargstr (specname)
		call pargd (ra)
		call pargd (dec)
 	    }
	else {
	    call sprintf(text, SZ_LINE, "Object: %s RA: INDEF DEC: INDEF")
		call pargstr (specname)
	    }
#	call htsize (0.8)
 	call gtext (gp, 0.025, 0.91, text, "f=r;q=m;s=0.8")
                        
#  Switch windows and put in numbers from program     

	call gseti (gp, G_WCS, 3)
	call gswind (gp, 0.0, 1.0, 0.0, 1.0)
 	call gsview (gp, 0.67, 1.0, 0.0, 1.0)
	lineno = 0.96

#  Date and time of observation
	mm = INDEFI
	dd = INDEFI
	yyyy = INDEFI
	call imgdate (image, "DATE-OBS", mm, dd, yyyy)
	call sprintf(text,SZ_LINE,"%02d-%3.3s-%04d %011.2h")
	call pargi (dd)
	call pargstr (month[(mm - 1) * SZ_MONTH + 1])
	call pargi (yyyy)
	call pargd (ut)
	call gtext (gp, 0.0, lineno, text, "f=r;q=m")

#  Heliocentric velocity correction
	switch (svcor) {
	    case HCV:
		call sprintf(text,SZ_LINE, "Object HCV: %7.3f")
		call pargd (spechcv)
	    case FBCV:
		if (specvb) {
		    call sprintf(text,SZ_LINE, "Object file BCV: %7.3f")
		    call pargd (spechcv)
		    }
		else {
		    call sprintf(text,SZ_LINE, "Object file HCV: %7.3f")
		    call pargd (spechcv)
		    }
	    case FHCV:
		if (specvb) {
		    call sprintf(text,SZ_LINE, "Object file BCV: %7.3f")
		    call pargd (spechcv)
		    }
		else {
		    call sprintf(text,SZ_LINE, "Object file HCV: %7.3f")
		    call pargd (spechcv)
		    }
	    case BCV: 
		call sprintf(text,SZ_LINE, "Object BCV: %7.3f")
		call pargd (spechcv)
	    default: 
		call sprintf(text,SZ_LINE, "Object BCV:    0.000")
	    }
	lineno = lineno - 0.04
	call gtext (gp, 0.0, lineno, text, "f=r;q=m")

#  Wavelength limits
       	call sprintf(text, SZ_LINE, "lambda: %.1f %.1f")
	call pargd (strtwav)
	call pargd (finwav)
	lineno = lineno - 0.04
	call gtext (gp, 0.0, lineno, text, "f=r;q=m")

#  Number of bins and tshift if nonzero
	call sprintf(text, SZ_LINE, "nbins: %d")
	    call pargi (npts)
	if (zpad)
	    call strcat ("x2",text,SZ_LINE)
	if (tshift != 0.0) {
	    call sprintf(text1, SZ_LINE, "  Tsh: %6.2f")
		call pargd (tshift)
	    call strcat (text1,text,SZ_LINE)
	    }
	if (tschop[itmax])
	    call strcat (" -em",text,SZ_LINE)
	if (tachop[itmax])
	    call strcat (" -ab",text,SZ_LINE)
	if (!tscont[itmax])
	    call strcat (" +tc",text,SZ_LINE)
	else if (tdivcon[itmax])
	    call strcat (" divc",text,SZ_LINE)
	lineno = lineno - 0.04
	call gtext (gp, 0.0, lineno, text, "f=r;q=m")

#  Transform filter limits
	call sprintf(text, SZ_LINE, "Filter: %d %d %d %d")
	call pargi(lo)	
	call pargi(toplo)
	call pargi(topnrn)
	call pargi(nrun)
	lineno = lineno - 0.04
	call gtext (gp, 0.0, lineno, text, "f=r;q=m")

#  Apodization 
	call sprintf(text, SZ_LINE, "Frac. endmask: %.2f")        
	call pargr (han)
	lineno = lineno - 0.04
	call gtext (gp, 0.0, lineno, text, "f=r;q=m")

#  Template information
	call gsetr (gp, G_TXSIZE, 0.65)

#  Sort templates by R-value
	do i = 1, ntemp {
	    itr[i] = i
	    }
	call gqsort (itr,ntemp,rcomp,czr)

#  Template velocities and velocity shift from template
	if (ntemp < 6) {
	    lineno = lineno - 0.04
	    call sprintf (text, SZ_LINE, "Template   vel.    hcv   peak")
	    call gtext (gp, 0.0, lineno, text,"f=r;q=m")
	    do j = 1, ntemp {
		i = itr[j]
		tmpvel = tempvel[i] + tempshift[i] + tvshift[i]
		call sprintf(text, SZ_LINE, "%-7.7s %7.3f %7.3f %6.2f")
		    call pargstr (tempid[1,i])
		    call pargd (tmpvel)
		    call pargd (temphcv[i])
		    call pargd (cz[i])
		lineno = lineno - 0.04
		call gtext (gp, 0.0, lineno, text, "f=r;q=m")
		}
	    }

#  Corrected heliocentric velocity and error
	lineno = lineno - 0.06
	call sprintf (text, SZ_LINE, "Template  CZ    error    R")
#	call htsize (0.7)
	call gtext (gp, 0.0, lineno, text, "f=r;q=m;s=0.7")
	if (ntemp > 10)
	    ntempx = 10
	else
	    ntempx = ntemp
	do j = 1, ntempx {
	    i = itr[j]
	    call sprintf (text, SZ_LINE, "%-7.7s %7.3f %7.3f %5.2f ")
		call pargstr (tempid[1,i])
		call pargd (zvel[i])
		call pargd (czerr[i])
		call pargd (czr[i])
	    if (toverlap[i])
		call strcat ("+w",text,SZ_LINE)
	    if (tschop[i])
		call strcat ("-e",text,SZ_LINE)
	    if (tachop[i])
		call strcat ("-a",text,SZ_LINE)
	    if (tempfilt[i] == 1)
		call strcat ("-f",text,SZ_LINE)
	    else if (tempfilt[i] == 2)
		call strcat ("+h",text,SZ_LINE)
	    else if (tempfilt[i] == 3)
		call strcat ("-f+h",text,SZ_LINE)
	    if (!tscont[i])
		call strcat ("+c",text,SZ_LINE)
	    else if (tdivcon[i])
		call strcat ("/c",text,SZ_LINE)
	    lineno = lineno - 0.04
	    call gtext (gp, 0.0, lineno, text, "f=r;q=m")
	    }                                                    
	call gflush(gp)
end                         


int procedure rcomp (rvalue, itemp1, itemp2)

double	rvalue[ARB]
int	itemp1
int	itemp2

begin
	if (rvalue[itemp1] < rvalue[itemp2])
	    return (1)
	else if (rvalue[itemp1] > rvalue[itemp2])
	    return (-1)
	else
	    return (0)
end
 
# June	1988	Stephen Levine (madraf::levine)
#               alter peak window for graph -- Morsey
# April	1990	Doug Mink (mink@cfa.harvard.edu)
#		alter display font sizes so everything fits
#		alter range for velocity plot
# Aug	1990	Add multiple peak fits and error calcs
# Oct	1990	Add file BCV or HCV
# June	1991	Simplify code
# July	1991	Scale data plot from 0
# Sept	1991	Make device a variable
# Sept	1991	Add template velocity shift
# Nov 20 1991	Shrink axis tick mark labels
# Dec  3 1991	Compute default width of velocity plot using peak width

# Feb 14 1992	Print mspec instead of eline for multispec spectra
# 		Print only result line if >5 templates
# Apr 20 1992	Pass mspec as argument
# Apr 21 1992	Change xcplot to xcorplot and spplot to specplot
#		Rename lsretup to openplot and lsrend to closeplot
# Aug 17 1992	Use htext to use Hershey fonts instead of ugly IRAF font
# Nov 24 1992	Go back to gtext instead of htext
# Dec  1 1992	If correlation error is near zero, set plot width to 300 km/s

# May 20 1993	Use different variable name for corrected template velocity
# Aug  9 1993	Plot spectrum within parameter limits if given

# Mar 23 1994	Print template id instead of object
# Apr 13 1994	Drop c0 as it is not used
# Jun 23 1994	Pass velocities in fquot insted of getim labelled common
# Aug  3 1994	Change common and header from fquot to rvsao
# Aug 15 1994	Move scaling and plot device handling to Util/plotutil.x
# Aug 16 1994	Fix zero-padding bug
# Aug 17 1994	Rename XCIPLOT for consistency
# Aug 19 1994	Add emission line chopping flag per template
# Dec 14 1994	Add correlation filter flag per template
# Dec 15 1994	Drop peak mode from summary line
# Dec 19 1994	Add high filter flag per template

# Jan 11 1995	Add additional filter flag value
# Mar 13 1995	Add notes for absorption line removal
# Mar 15 1995	Add note for template continuum left in
# May 10 1995	Print info for first 12 templates only
# May 10 1995	Sort templates by R-value for output
# May 15 1995	Note if continuum has been divided instead of subtracted
# Jun 19 1995	Use SPECID instead of making it up
# Jun 26 1995	Note if full overlap region is being used per template
# Jun 26 1995	Drop spaces in flags
# Jul  7 1995	Print BCV if file HCV not found
# Jul 13 1995	Get Julian Date from JULDATE, not JULDAY
# Jul 13 1995	Get UT from JULDATE
# Aug 22 1995	Use absolute value of PKFRAC
